package eu.jonahbauer.survey.controller;

import eu.jonahbauer.survey.persistence.entity.Survey;
import eu.jonahbauer.survey.persistence.entity.User;
import eu.jonahbauer.survey.persistence.repository.SurveyRepository;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;

@Controller
@Getter(AccessLevel.PRIVATE)
@RequiredArgsConstructor
public class IndexController {
    private final SurveyRepository surveyRepository;

    @GetMapping({"/index", "/", "/index.html"})
    public String index(Authentication authentication, Model model) {
        Pageable pageableRecent = PageRequest.of(0, 8, Sort.by(Sort.Direction.DESC, "created"));
        Page<Survey> recentSurveys = getSurveyRepository().findAll(pageableRecent);

        Pageable pageableHot = PageRequest.of(0, 8);
        Page<Survey> hotSurveys = getSurveyRepository().findAllSortedBySubmissionCountDesc(pageableHot);

        model.addAttribute("recent", recentSurveys);
        model.addAttribute("hot", hotSurveys);

        if (authentication != null) {
            User user = (User) authentication.getPrincipal();
            Pageable pageableOwn = PageRequest.of(0, 8, Sort.by(Sort.Direction.DESC, "created"));
            Page<Survey> ownSurveys = getSurveyRepository().findAllByOwner(user, pageableOwn);
            model.addAttribute("own", ownSurveys);
        }

        return "index";
    }
}
