package eu.jonahbauer.survey.controller;

import eu.jonahbauer.survey.model.RegistrationForm;
import eu.jonahbauer.survey.persistence.entity.User;
import eu.jonahbauer.survey.persistence.repository.UserRepository;
import jakarta.validation.Valid;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;

@Controller
@Getter(AccessLevel.PRIVATE)
@RequiredArgsConstructor
public class RegisterController {
    private final UserRepository userRepository;
    private final PasswordEncoder passwordEncoder;

    @GetMapping("/register")
    public String get(Model model) {
        model.addAttribute("form", new RegistrationForm());
        return "register";
    }

    @PostMapping("/register")
    public String post(@Valid @ModelAttribute("form") RegistrationForm form, BindingResult bindingResult) {
        User user = getUserRepository().findByUsername(form.getUsername());

        if (user != null) {
            bindingResult.addError(new ObjectError("form", new String[] {"registration.username.unique"}, null, null));
        }

        if (bindingResult.hasErrors()) {
            return "register";
        } else {
            user = new User();
            user.setUsername(form.getUsername());
            user.setPassword(getPasswordEncoder().encode(form.getPassword()));
            form.setPassword(null);

            getUserRepository().save(user);

            return "redirect:/login?success";
        }
    }
}
