package eu.jonahbauer.survey.controller.survey;

import eu.jonahbauer.survey.persistence.entity.Survey;
import eu.jonahbauer.survey.persistence.repository.SurveyRepository;
import eu.jonahbauer.survey.persistence.entity.User;
import jakarta.validation.constraints.NotNull;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.springframework.context.MessageSource;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.server.ResponseStatusException;

import java.util.Locale;

@Controller
@Getter(AccessLevel.PRIVATE)
@RequiredArgsConstructor
public class SurveyListController {
    private final SurveyRepository surveyRepository;
    private final MessageSource messageSource;

    @GetMapping("/survey/recent")
    public String recent(Model model, Locale locale) {
        Pageable pageable = PageRequest.of(0, 20, Sort.by(Sort.Direction.DESC, "created"));
        Page<Survey> surveys = getSurveyRepository().findAll(pageable);

        model.addAttribute("surveys", surveys);
        model.addAttribute("title", getMessageSource().getMessage("index.heading.recent", null, locale));
        model.addAttribute("pattern", "list.data_pattern.recent");

        return "survey/list";
    }

    @GetMapping("/survey/own")
    public String own(@NotNull Authentication authentication, Model model, Locale locale) {
        if (authentication == null) {
            throw new ResponseStatusException(HttpStatus.UNAUTHORIZED);
        }

        Pageable pageable = PageRequest.of(0, 20, Sort.by(Sort.Direction.DESC, "created"));
        Page<Survey> surveys = getSurveyRepository().findAllByOwner((User) authentication.getPrincipal(), pageable);

        model.addAttribute("surveys", surveys);
        model.addAttribute("title", getMessageSource().getMessage("index.heading.own", null, locale));
        model.addAttribute("pattern", "list.data_pattern.recent");

        return "survey/list";
    }

    @GetMapping("/survey/hot")
    public String hot(Model model, Locale locale) {
        Pageable pageable = PageRequest.of(0, 20);
        Page<Survey> surveys = getSurveyRepository().findAllSortedBySubmissionCountDesc(pageable);

        model.addAttribute("surveys", surveys);
        model.addAttribute("title", getMessageSource().getMessage("index.heading.hot", null, locale));
        model.addAttribute("pattern", "list.data_pattern.hot");

        return "survey/list";
    }
}
