package eu.jonahbauer.survey.persistence.entity;

import jakarta.persistence.*;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.Hibernate;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

@Entity
@Data
@Table(name = "answer", uniqueConstraints = {
        @UniqueConstraint(name = "uq_answer_question_number", columnNames = {"question", "number"})
})
@EqualsAndHashCode(of = "id")
public class Answer implements Comparable<Answer> {
    @Id
    @Column(name = "id", nullable = false)
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Integer id;

    /**
     * The answer text.
     */
    @Column(name = "text", nullable = false)
    @NotBlank
    private String text;

    /**
     * The consecutive number of this answer as part of a question.
     */
    @Column(name = "number", nullable = false)
    @NotNull
    private Integer number;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "question", nullable = false, foreignKey = @ForeignKey(name = "fk_answer_question"))
    @OnDelete(action = OnDeleteAction.CASCADE)
    @NotNull
    private Question question;

    @OneToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "id")
    private AnswerEvaluation evaluation;

    @Override
    public int compareTo(Answer o) {
        return this.getNumber().compareTo(o.getNumber());
    }

    public String toString() {
        return "Answer(id=" + this.getId() +
                ", text=" + this.getText() +
                ", number=" + this.getNumber() +
                (Hibernate.isInitialized(getEvaluation()) ? ", evaluation=" + this.getEvaluation() : "") +
                ")";
    }
}
