package eu.jonahbauer.survey.persistence.entity;

import jakarta.persistence.*;
import jakarta.validation.constraints.NotNull;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.Hibernate;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;


import java.time.Instant;
import java.util.Set;

/**
 * A submission to a survey modelled as a set of selected answers.
 */
@Entity
@Data
@Table(name = "submission")
@EqualsAndHashCode(of = "id")
public class Submission {
    @Id
    @Column(name = "id", nullable = false)
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Integer id;

    @Column(name = "created", nullable = false, insertable = false, updatable = false)
    private Instant created;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "survey", nullable = false, foreignKey = @ForeignKey(name = "fk_submission_survey"))
    @OnDelete(action = OnDeleteAction.CASCADE)
    @NotNull
    private Survey survey;

    /**
     * The set of selected answers. The answers must all belong to the same survey as this submission and the number
     * of answers per question must be within the bounds set by {@link Question#getMinAnswers()} and
     * {@link Question#getMaxAnswers()}.
     */
    @ManyToMany(fetch = FetchType.LAZY)
    @JoinTable(
            name = "submission_answer",
            joinColumns = @JoinColumn(
                    name = "submission", referencedColumnName = "id", nullable = false,
                    foreignKey = @ForeignKey(name = "fk_submission_answer_submission")
            ),
            inverseJoinColumns = @JoinColumn(
                    name = "answer", referencedColumnName = "id", nullable = false,
                    foreignKey = @ForeignKey(name = "fk_submission_answer_answer")
            )
    )
    private Set<Answer> answers;

    public String toString() {
        return "Submission(id=" + this.getId() +
                ", created=" + this.getCreated() +
                ", survey=" + this.getSurvey() +
                (Hibernate.isInitialized(getAnswers()) ? ", answers=" + this.getAnswers() : "") +
                ")";
    }
}
