package eu.jonahbauer.survey.persistence.repository;

import eu.jonahbauer.survey.persistence.entity.Survey;
import eu.jonahbauer.survey.persistence.entity.User;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.EntityGraph;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

public interface SurveyRepository extends JpaRepository<Survey, Integer> {
    @EntityGraph(attributePaths = {"questions.answers"})
    Survey findDetailedById(Integer id);

    @EntityGraph(attributePaths = {"questions.answers.evaluation", "owner", "submissions"})
    Survey findEvaluationById(Integer id);

    @Query(
            value = "select new Survey(s.id, s.title, s.summary, s.created, count(sub.id)) from Survey s left join s.submissions sub group by s order by count(sub.id) desc",
            countQuery = "select count(Survey) from Survey"
    )
    Page<Survey> findAllSortedBySubmissionCountDesc(Pageable pageable);

    Page<Survey> findAllByOwner(User owner, Pageable pageable);

    @Query("select count(sub.id) from Survey s left join s.submissions sub where s.id = :id")
    long getSubmissionCountById(@Param("id") Integer id);
}
