package eu.jonahbauer.survey.spring;

import jakarta.validation.constraints.NotNull;
import lombok.experimental.Delegate;
import org.springframework.context.annotation.Configuration;
import org.thymeleaf.context.IExpressionContext;
import org.thymeleaf.dialect.AbstractDialect;
import org.thymeleaf.dialect.IExpressionObjectDialect;
import org.thymeleaf.expression.IExpressionObjectFactory;
import org.thymeleaf.expression.Temporals;
import org.thymeleaf.spring6.SpringTemplateEngine;
import org.thymeleaf.standard.expression.StandardExpressionObjectFactory;

import java.time.Instant;
import java.time.ZoneId;
import java.util.Date;
import java.util.Locale;
import java.util.Set;

@Configuration
public class ThymeleafConfig {
    public ThymeleafConfig(SpringTemplateEngine templateEngine) {
        templateEngine.setRenderHiddenMarkersBeforeCheckboxes(true);
        templateEngine.addDialect(new CustomDialect());
    }

    public static final class CustomDialect extends AbstractDialect implements IExpressionObjectDialect {
        private final IExpressionObjectFactory CUSTOM_EXPRESSION_OBJECT_FACTORY = new CustomExpressionObjectFactory();

        private CustomDialect() {
            super("custom");
        }

        @Override
        public IExpressionObjectFactory getExpressionObjectFactory() {
            return CUSTOM_EXPRESSION_OBJECT_FACTORY;
        }
    }

    public static final class CustomExpressionObjectFactory implements IExpressionObjectFactory {
        private static final Set<String> ALL_EXPRESSION_OBJECT_NAMES = Set.of(StandardExpressionObjectFactory.TEMPORALS_EXPRESSION_OBJECT_NAME);

        @Override
        public Set<String> getAllExpressionObjectNames() {
            return ALL_EXPRESSION_OBJECT_NAMES;
        }

        @Override
        public Object buildObject(IExpressionContext context, String expressionObjectName) {
            if (StandardExpressionObjectFactory.TEMPORALS_EXPRESSION_OBJECT_NAME.equals(expressionObjectName)) {
                return new CustomTemporals(context.getLocale());
            } else {
                return null;
            }
        }

        @Override
        public boolean isCacheable(String expressionObjectName) {
            return true;
        }
    }

    public static final class CustomTemporals {
        @Delegate
        private final Temporals temporals;

        public CustomTemporals(@NotNull Locale locale) {
            this(locale, ZoneId.systemDefault());
        }

        public CustomTemporals(@NotNull Locale locale, ZoneId zoneId) {
            this.temporals = new Temporals(locale, zoneId);
        }

        public Date asDate(Instant instant) {
            return Date.from(instant);
        }
    }
}
