CREATE TABLE `user`
(
    `id`       INTEGER      NOT NULL AUTO_INCREMENT,
    `password` VARCHAR(255) NOT NULL,
    `username` VARCHAR(255) NOT NULL,
    PRIMARY KEY (`id`),
    CONSTRAINT `uq_user_username` UNIQUE (`username`)
);

CREATE TABLE `survey`
(
    `id`      INTEGER      NOT NULL AUTO_INCREMENT,
    `created` TIMESTAMP    NOT NULL DEFAULT CURRENT_TIMESTAMP,
    `summary` VARCHAR(255) NOT NULL,
    `title`   VARCHAR(255) NOT NULL,
    `owner`   INTEGER      NOT NULL,
    PRIMARY KEY (`id`),
    CONSTRAINT `fk_survey_owner` FOREIGN KEY (`owner`) REFERENCES `user` (`id`)
        ON DELETE CASCADE
);

CREATE TABLE `question`
(
    `id`          INTEGER      NOT NULL AUTO_INCREMENT,
    `max_answers` INTEGER      NOT NULL CHECK (`max_answers` >= 1),
    `min_answers` INTEGER      NOT NULL CHECK (`min_answers` >= 0),
    `number`      INTEGER      NOT NULL CHECK (`number` >= 0),
    `text`        VARCHAR(255) NOT NULL,
    `survey`      INTEGER      NOT NULL,
    PRIMARY KEY (`id`),
    CONSTRAINT `uq_question_survey_number` UNIQUE (`survey`, `number`),
    CONSTRAINT `fk_question_survey` FOREIGN KEY (`survey`) REFERENCES `survey` (`id`)
        ON DELETE CASCADE
);

CREATE TABLE `answer`
(
    `id`       INTEGER      NOT NULL AUTO_INCREMENT,
    `number`   INTEGER      NOT NULL,
    `text`     VARCHAR(255) NOT NULL,
    `question` INTEGER      NOT NULL,
    PRIMARY KEY (`id`),
    CONSTRAINT `uq_answer_question_number` UNIQUE (`question`, `number`),
    CONSTRAINT `fk_answer_question` FOREIGN KEY (`question`) REFERENCES `question` (`id`)
        ON DELETE CASCADE
);

CREATE TABLE `submission`
(
    `id`      INTEGER   NOT NULL AUTO_INCREMENT,
    `created` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    `survey`  INTEGER   NOT NULL,
    PRIMARY KEY (`id`),
    CONSTRAINT `fk_submission_survey` FOREIGN KEY (`survey`) REFERENCES `survey` (`id`)
        ON DELETE CASCADE
);

CREATE TABLE `submission_answer`
(
    `submission` INTEGER NOT NULL,
    `answer`     INTEGER NOT NULL,
    PRIMARY KEY (`submission`, `answer`),
    CONSTRAINT `fk_submission_answer_answer` FOREIGN KEY (`answer`) REFERENCES `answer` (`id`)
        ON DELETE CASCADE,
    CONSTRAINT `fk_submission_answer_submission` FOREIGN KEY (`submission`) REFERENCES `submission` (`id`)
        ON DELETE CASCADE
);

CREATE VIEW `answer_evaluation` AS
(
    SELECT `a`.`id` AS `answer`, COUNT(`sa`.`answer`) AS `count`
    FROM `answer` `a`
         LEFT JOIN `submission_answer` `sa` ON `a`.`id` = `sa`.`answer`
    GROUP BY `a`.`id`
);