const max_questions_per_survey = 20;
const max_answers_per_question = 20;
const answer_count_pattern = document.getElementById('answer-count-text').innerText;

function addQuestion(survey) {
    if (survey.querySelectorAll(".question").length >= max_questions_per_survey) return;

    // load question from template
    let template = document.getElementById('question-template');
    let questionNode = template.content.cloneNode(true);

    setupSlider(questionNode.querySelector('.double-slider'));

    // add answer
    addAnswer(questionNode);

    // append to dom
    survey.insertBefore(questionNode, survey.querySelector('.survey > footer'));

    // set visibility of "add question" button
    if (survey.children.length >= max_questions_per_survey) {
        document.getElementById('add-question').style.display = 'none';
    } else {
        document.getElementById('add-question').style.display = null;
    }

    return questionNode;
}

function removeQuestion(question) {
    if (!question.matches(':only-of-type')) {
        let survey = question.closest('.survey');

        survey.removeChild(question);

        if (survey.children.length >= max_questions_per_survey) {
            document.getElementById('add-question').style.display = 'none';
        } else {
            document.getElementById('add-question').style.display = null;
        }
    }
}

function addAnswer(question) {
    if (question.querySelectorAll(".answer").length >= max_answers_per_question) return;

    // load template
    let template = document.getElementById('answer-template');
    let answerNode = template.content.cloneNode(true);

    // add to dom
    question.querySelector('.answers ol').appendChild(answerNode);

    updateAnswerCount(question);

    return answerNode;
}

function addAnswerOnInput(question, answer) {
    if (answer.matches(".new")) {
        answer.children[0].placeholder = 'antwortmöglichkeit';
        answer.children[0].oninput = '';
        answer.classList.toggle('new', false);

        let question = answer.closest('.question');

        if (addAnswer(question) === undefined) {
            updateAnswerCount(question);
        }
    } else {
        answer.children[0].oninput = '';
    }
}

function removeAnswer(answer) {
    if (!answer.matches(".new")) {
        let ol = answer.parentElement;

        ol.removeChild(answer);

        updateAnswerCount(ol.closest('.question'));
    }
}

function updateAnswerCount(question) {
    let slider = question.querySelector('.double-slider');

    let answerCount = question.querySelectorAll('.answer:not(.new)').length;

    if (answerCount < max_answers_per_question && question.querySelector('.answer.new') == null) {
        addAnswer(question);
    }

    slider.querySelector('input.left').max = answerCount;
    slider.querySelector('input.right').max = answerCount;
    updateSlider(slider);

    updateAnswerCountText(question);
}

function updateAnswerCountText(question) {
    let text = question.querySelector('.answer-count-text');

    let slider = question.querySelector('.double-slider');
    let left = slider.querySelector('.left').value;
    let right = slider.querySelector('.right').value;

    question.querySelector('.answers').classList.toggle('multiple', right !== '1' || left !== '1');

    text.innerText = answer_count_pattern.replace('{min}', left).replace('{max}', right);
}

function prepareNames(survey) {
    let questions = survey.querySelectorAll('.question');

    questions.forEach((question, qindex) => {
        let qBaseName = `questions[${qindex}]`;

        question.querySelector('header > input').name = qBaseName + '.text';
        question.querySelector('footer > .double-slider > .left').name = qBaseName + '.minAnswers';
        question.querySelector('footer > .double-slider > .right').name = qBaseName + '.maxAnswers';

        question.querySelectorAll('.answers .answer:not(.new)').forEach((answer, aindex) => {
            answer.querySelector('input').name = qBaseName + `.answers[${aindex}]`;
        })
    })
}

window.addEventListener('load', () => {
    document.getElementById('survey-form')
            .addEventListener('submit', () => {
                prepareNames(document.getElementById('survey-form'));
            });
});